@@ -23,9 +23,9 @@ import h5py
 import logging
 from pyarrow.dataset import field as ffield
 from pyarrow import dataset as ds
-from typing import Union, Iterable
+from typing import Iterable
 from pathlib import Path
-from .tools import bisect_left, bisect_right, to_intdate, read_version_config
+from .tools import bisect_left, bisect_right, to_intdate, read_version_config, read_datapath
 from .config import data_path, cal_config
 
 markets = ["zz500", "hs300", "zz800", "zz1000", "all"]
@@ -39,7 +39,8 @@ logger.addHandler(stream_handler)
 class BaseProvider:
     """ data prov"""
 
-    def calendar(self, start_time=None, end_time=None, freq="Tday"):
+    @staticmethod
+    def calendar(start_time=None, end_time=None, freq="Tday"):
         """
         :param start_time
         :param end_time
@@ -48,7 +49,8 @@ class BaseProvider:
         """
         return Cal.calendar(start_time, end_time, freq)
 
-    def list_instruments(self, instruments, start_time=None, end_time=None, freq="Tday", as_list=False):
+    @staticmethod
+    def list_instruments(instruments, start_time=None, end_time=None, freq="Tday", as_list=False):
         """
         :param instruments: instruments list or market (ei. "zz500", "hs300", "zz800", "zz1000", "all"
         :param start_time:
@@ -59,14 +61,16 @@ class BaseProvider:
         """
         return Inst.list_instruments(instruments, start_time, end_time, freq, as_list)
 
-    def list_features(self, freq="*"):
+    @staticmethod
+    def list_features(freq="*"):
         """
         :param freq: includdes Aday for natural day; Tday for trading day; report_period for financial statement
         :return: list of features
         """
         return list_features(freq)
 
-    def feature_info(self, field, freq):
+    @staticmethod
+    def feature_info(field, freq):
         """
         return the config of the field
         :param field: features str
@@ -75,7 +79,8 @@ class BaseProvider:
         """
         return FeatureD.feature_info(field, freq)
 
-    def features(self, fields, start_time=None, end_time=None, freq="Tday", return_mat=False, instruments="all"):
+    @staticmethod
+    def features(fields, start_time=None, end_time=None, freq="Tday", return_mat=False, instruments="all"):
         """
         :param instruments: instruments list or market (ei. "zz500", "hs300", "zz800", "zz1000", "all"
         :param fields: features str or list
@@ -92,7 +97,8 @@ class BaseProvider:
         fields = list(fields)  # In case of tuple.
         return FeatureD.features(instruments, fields, start_time, end_time, freq, return_mat)
 
-    def features_mmat(self, fields, start_time=None, end_time=None, freq="Tday", instruments="all"):
+    @staticmethod
+    def features_mmat(fields, start_time=None, end_time=None, freq="Tday", instruments="all"):
         """
         :param instruments: instruments list or market (ei. "zz500", "hs300", "zz800", "zz1000", "all"
         :param fields: features str or list
@@ -141,20 +147,66 @@ class BaseProvider:
         """
         return self.features(fields, start_time, end_time, "report_period", return_mat, instruments)
 
-    def list_datasets(self):
+    @staticmethod
+    def alphas(fields, start_time=None, end_time=None, freq="Tday", return_mat=False, instruments="all"):
+        """
+        :param instruments: instruments list or market (ei. "zz500", "hs300", "zz800", "zz1000", "all"
+        :param fields: features str or list
+        :param start_time:
+        :param end_time:
+        :param freq: includdes Aday for natural day; Tday for trading day; report_period for financial statement
+        :param return_mat: if return_mat, return nd-ns numpy array
+        :return:
+        """
+        if isinstance(fields, str):
+            fields = [fields, ]
+        if freq == "report_period":
+            fields += ["ann_date", ]
+        fields = list(fields)  # In case of tuple.
+        return AlphaD.features(instruments, fields, start_time, end_time, freq, return_mat)
+
+    @staticmethod
+    def alphas_mmat(fields, start_time=None, end_time=None, freq="Tday", instruments="all"):
+        """
+        :param instruments: instruments list or market (ei. "zz500", "hs300", "zz800", "zz1000", "all"
+        :param fields: features str or list
+        :param start_time:
+        :param end_time:
+        :param freq: includdes Aday for natural day; Tday for trading day; report_period for financial statement
+        :return: DataFrames that may have different shapes
+        """
+        if isinstance(fields, str):
+            fields = [fields, ]
+        if freq == "report_period":
+            fields += ["ann_date", ]
+        fields = list(fields)  # In case of tuple.
+        return AlphaD.features_mmat(instruments, fields, start_time, end_time, freq)
+
+    @staticmethod
+    def list_alphas(freq="*"):
+        """
+        :param freq: includdes Aday for natural day; Tday for trading day; report_period for financial statement
+        :return: list of features
+        """
+        return list_alphas(freq)
+
+    @staticmethod
+    def list_datasets():
         """
         :return:  list of dataset name
         """
         return list_datasets()
 
-    def dataset_info(self, dataset):
+    @staticmethod
+    def dataset_info(dataset):
         """
         :param dataset: dataset name
         :return: the columns with dtype info of the dataset
         """
         return DatasetD.dataset_info(dataset)
 
-    def datasets(self, dataset, instruments="all", fields=None, start_time=None, end_time=None):
+    @staticmethod
+    def datasets(dataset, instruments="all", fields=None, start_time=None, end_time=None):
         """
         :param instruments: instruments list or market (ei. "zz500", "hs300", "zz800", "zz1000", "all"
         :param fields:  features str or list
@@ -171,7 +223,8 @@ class BaseProvider:
         fields = list(fields)
         return DatasetD.datasets(instruments, fields, start_time, end_time, dataset)
 
-    def datasets_fl(self, dataset, fields=None, filt=None, **kwargs):
+    @staticmethod
+    def datasets_fl(dataset, fields=None, filt=None, **kwargs):
         """
         :param filt: conditions to filt the dataset
         :param fields:
@@ -187,7 +240,8 @@ class BaseProvider:
         fields = list(fields)
         return DatasetD.datasets_fl(filt, fields, dataset, **kwargs)
 
-    def version_info(self):
+    @staticmethod
+    def version_info():
         return read_version_config()
 
 
@@ -293,7 +347,7 @@ class CalendarProvider:
         locations = self.cal_loc(dt, mode, freq)
         return _calendar[locations]
 
-    def cal_loc(self, dt: Iterable, mode: str = "b", freq: str = "Tday"):
+    def cal_loc(self, dt, mode: str = "b", freq: str = "Tday"):
         """
         :param dt: int or iterable
         :param mode: f for forward or b for backward if dt not in calendar index
@@ -301,22 +355,31 @@ class CalendarProvider:
         :return: int or nparray
         """
         calendar, calendar_index = self._get_calendar(freq=freq)
+        if isinstance(dt, str):
+            try:
+                dt = int(dt)
+            except TypeError:
+                raise TypeError("dt cannot be safely convert to int")
         if isinstance(dt, Iterable):
             index = np.array([self.cal_loc(dt_i, mode, freq) for dt_i in dt])
-        elif isinstance(dt, (int, np.int)):
-            if dt not in calendar_index:
-                try:
-                    if mode == "b":
-                        dt = calendar[bisect_right(calendar, dt) - 1]
-                    else:
-                        dt = calendar[bisect_left(calendar, dt)]
-                except IndexError:
-                    raise IndexError(
-                        f"`start_time` doesnt exist in {freq} calendar`"
-                    )
-            index = calendar_index[dt]
+        elif np.isnan(dt):
+            return np.nan
         else:
-            raise TypeError("dt is not int or iterable")
+            try:
+                dt = int(dt)
+                if dt not in calendar_index:
+                    try:
+                        if mode == "b":
+                            dt = calendar[bisect_right(calendar, dt) - 1]
+                        else:
+                            dt = calendar[bisect_left(calendar, dt)]
+                    except IndexError:
+                        raise IndexError(
+                            f"`start_time` doesnt exist in {freq} calendar`"
+                        )
+                index = calendar_index[dt]
+            except:
+                raise TypeError("dt is not int or iterable")
         return index
 
 
@@ -579,6 +642,18 @@ class FeatureProvider(BaseDataProvider):
         return json_cfg
 
 
+class AlphaProvider(FeatureProvider):
+
+    @property
+    def _uri_data(self):
+        return os.path.join(data_path, "alphas", "{}", "{}.h5")
+
+    @property
+    def _uri_cfg(self):
+        """Static feature file uri."""
+        return os.path.join(data_path, "alphas", "{}", "{}_config.json")
+
+
 class DatasetProvider(BaseDataProvider):
     """Dataset provider class: Provide Dataset data."""
 
@@ -704,6 +779,12 @@ def list_features(freq="*"):
     return features
 
 
+def list_alphas(freq="*"):
+    features = (Path(data_path) / "alphas").glob(f"**/{freq}/*.h5")
+    features = [i.name.replace(".h5", "") for i in features]
+    return features
+
+
 def list_datasets():
     datasets = (Path(data_path) / "datasets").glob("*")
     datasets = [i.name for i in datasets if i.is_dir()]
@@ -779,8 +860,10 @@ def parse_field(field):
 
 DatasetD = DatasetProvider()
 FeatureD = FeatureProvider()
+AlphaD = AlphaProvider()
 Cal = CalendarProvider()
 Inst = InstrumentProvider()
 D = BaseProvider()
 
 kernels = 20
+print("using", read_datapath())
\ No newline at end of file
