@@ -35,7 +35,7 @@ class FeatureUpdater:
         self.freq = freq
         self.start_time = start_time
         self.end_time = end_time
-        self.is_alpha = is_alpha
+        self.save_primarykey = is_alpha
 
     @property
     def _uri_data(self):
@@ -70,7 +70,7 @@ class FeatureUpdater:
         if config is None:
             config = {}
         value_dict = Mdf.to_dict("series")
-        if self.is_alpha:
+        if self.save_primarykey:
             value_dict.pop(self.cal_config[self.freq]["col_name"])
         for field, values in value_dict.items():
             config_i = config.get(field, None)
@@ -90,14 +90,28 @@ class FeatureUpdater:
         return Mdf.set_index(partition_by)
 
 
-class FeatureSaver(FeatureUpdater):
+class AlphaUpdater(FeatureUpdater):
+    """ save more complex alphas based on any time index of the api in, load from local with D.alphas"""
+    def __init__(self, stk_univ="all", start_time=None, end_time=None, freq="Tday", save_primarykey=False):
+        super(AlphaUpdater, self).__init__(stk_univ, start_time, end_time, freq, save_primarykey)
 
+    @property
+    def _uri_data(self):
+        """Static feature file uri."""
+        return os.path.join(data_path, "alphas", "{}", "{}.h5")
+
+    @property
+    def _uri_cfg(self):
+        """Static feature file uri."""
+        return os.path.join(data_path, "alphas", "{}", "{}_config.json")
+
+
+class FeatureSaver(FeatureUpdater):
+    """ save basic alphas based on complete time index of the api in, load from local with D.features"""
     def __init__(self, freq):
         super().__init__(freq=freq)
+        print("this class will be dropped in the future")
 
-    def save_alpha(self, alpha, alpha_name):
+    def save_alpha(self, alpha, alpha_name, config=None):
         alpha = alpha.rename(columns={"value": alpha_name})
-        self.save_values(alpha, self.cal_config[self.freq]["col_name"], "stockcode")
-
-
-print("res")
\ No newline at end of file
+        self.save_values(alpha, self.cal_config[self.freq]["col_name"], "stockcode", config)
